/******************************************************************************
 *
 * 
 *
 * Copyright (C) 1997-2006 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#ifndef TRANSLATORDECODER_H
#define TRANSLATORDECODER_H

#include <iconv.h>
#include "translator.h"

class TranslatorDecoder : public Translator
{
  public:

    TranslatorDecoder(Translator *translator) : m_translator(translator) 
    {
      m_cd = iconv_open("UTF-8",translator->idLanguageCharset()); 
    }
    virtual ~TranslatorDecoder() 
    { 
      if (m_cd) iconv_close(m_cd);
      delete m_translator; 
    }
   
    /*! This method is used to provide warning message that is displayed
     *  if the user chooses a language whose translation is not up to date. 
     *  It is implemented by the adapter classes.
     */
    virtual QCString updateNeededMessage() { return m_translator->updateNeededMessage(); }

    // Please, have a look at comments inside the translator_en.h file
    // to learn the meaning of the following methods.  The translator_en.h
    // file contains the TranslatorEnglish implementation, which is 
    // always up-to-date (by definition).
   
    // --- Language control methods -------------------
    
    QCString idLanguage()  
    { return m_translator->idLanguage(); }
    QCString latexLanguageSupportCommand()  
    { return m_translator->latexLanguageSupportCommand(); }
    QCString idLanguageCharset()  
    { return m_translator->idLanguageCharset(); }

    QCString decode(const QCString &s) 
    {
      if (m_cd==0) return s;
      int iSize        = s.length();
      int oSize        = iSize*4+1;
      QCString output(oSize);
      size_t iLeft     = iSize;
      size_t oLeft     = oSize;
      const char *iPtr = s.data();
      char *oPtr       = output.data();
      if (!iconv(m_cd,&iPtr,&iLeft,&oPtr,&oLeft))
      {
        oSize -= oLeft;
        output.resize(oSize+1);
        output.at(oSize+1)='\0';
        return output;
      }
      else
      {
        return s;
      }
    }

    // --- Language translation methods -------------------

    QCString trRelatedFunctions()  
    { return decode(m_translator->trRelatedFunctions()); }
    QCString trRelatedSubscript()  
    { return decode(m_translator->trRelatedSubscript()); }
    QCString trDetailedDescription()  
    { return decode(m_translator->trDetailedDescription()); }
    QCString trMemberTypedefDocumentation()  
    { return decode(m_translator->trMemberTypedefDocumentation()); }
    QCString trMemberEnumerationDocumentation()  
    { return decode(m_translator->trMemberEnumerationDocumentation()); }
    QCString trMemberFunctionDocumentation()  
    { return decode(m_translator->trMemberFunctionDocumentation()); }
    QCString trMemberDataDocumentation()  
    { return decode(m_translator->trMemberDataDocumentation()); }
    QCString trMore()  
    { return decode(m_translator->trMore()); }
    QCString trListOfAllMembers()  
    { return decode(m_translator->trListOfAllMembers()); }
    QCString trMemberList()  
    { return decode(m_translator->trMemberList()); }
    QCString trThisIsTheListOfAllMembers()  
    { return decode(m_translator->trThisIsTheListOfAllMembers()); }
    QCString trIncludingInheritedMembers()  
    { return decode(m_translator->trIncludingInheritedMembers()); }
    QCString trGeneratedAutomatically(const char *s)  
    { return decode(m_translator->trGeneratedAutomatically(s)); }
    QCString trEnumName()  
    { return decode(m_translator->trEnumName()); }
    QCString trEnumValue()  
    { return decode(m_translator->trEnumValue()); }
    QCString trDefinedIn()  
    { return decode(m_translator->trDefinedIn()); }

    // quick reference sections

    QCString trModules()  
    { return decode(m_translator->trModules()); }
    QCString trClassHierarchy()  
    { return decode(m_translator->trClassHierarchy()); }
    QCString trCompoundList()  
    { return decode(m_translator->trCompoundList()); }
    QCString trFileList()  
    { return decode(m_translator->trFileList()); }
    QCString trCompoundMembers()  
    { return decode(m_translator->trCompoundMembers()); }
    QCString trFileMembers()  
    { return decode(m_translator->trFileMembers()); }
    QCString trRelatedPages()  
    { return decode(m_translator->trRelatedPages()); }
    QCString trExamples()  
    { return decode(m_translator->trExamples()); }
    QCString trSearch()  
    { return decode(m_translator->trSearch()); }
    QCString trClassHierarchyDescription()  
    { return decode(m_translator->trClassHierarchyDescription()); }
    QCString trFileListDescription(bool extractAll)  
    { return decode(m_translator->trFileListDescription(extractAll)); }
    QCString trCompoundListDescription()  
    { return decode(m_translator->trCompoundListDescription()); }
    QCString trCompoundMembersDescription(bool extractAll)  
    { return decode(m_translator->trCompoundMembersDescription(extractAll)); }
    QCString trFileMembersDescription(bool extractAll)  
    { return decode(m_translator->trFileMembersDescription(extractAll)); }
    QCString trExamplesDescription()  
    { return decode(m_translator->trExamplesDescription()); }
    QCString trRelatedPagesDescription()  
    { return decode(m_translator->trRelatedPagesDescription()); }
    QCString trModulesDescription()  
    { return decode(m_translator->trModulesDescription()); }
    
    // index titles (the project name is prepended for these) 

    QCString trDocumentation()  
    { return decode(m_translator->trDocumentation()); }
    QCString trModuleIndex()  
    { return decode(m_translator->trModuleIndex()); }
    QCString trHierarchicalIndex()  
    { return decode(m_translator->trHierarchicalIndex()); }
    QCString trCompoundIndex()  
    { return decode(m_translator->trCompoundIndex()); }
    QCString trFileIndex()  
    { return decode(m_translator->trFileIndex()); } 
    QCString trModuleDocumentation()  
    { return decode(m_translator->trModuleDocumentation()); }
    QCString trClassDocumentation()  
    { return decode(m_translator->trClassDocumentation()); }
    QCString trFileDocumentation()  
    { return decode(m_translator->trFileDocumentation()); }
    QCString trExampleDocumentation()  
    { return decode(m_translator->trExampleDocumentation()); }
    QCString trPageDocumentation()  
    { return decode(m_translator->trPageDocumentation()); }
    QCString trReferenceManual()  
    { return decode(m_translator->trReferenceManual()); }
    QCString trDefines()  
    { return decode(m_translator->trDefines()); }
    QCString trFuncProtos()  
    { return decode(m_translator->trFuncProtos()); }
    QCString trTypedefs()  
    { return decode(m_translator->trTypedefs()); }
    QCString trEnumerations()  
    { return decode(m_translator->trEnumerations()); }
    QCString trFunctions()  
    { return decode(m_translator->trFunctions()); }
    QCString trVariables()  
    { return decode(m_translator->trVariables()); }
    QCString trEnumerationValues()  
    { return decode(m_translator->trEnumerationValues()); }
    QCString trDefineDocumentation()  
    { return decode(m_translator->trDefineDocumentation()); }
    QCString trFunctionPrototypeDocumentation()  
    { return decode(m_translator->trFunctionPrototypeDocumentation()); }
    QCString trTypedefDocumentation()  
    { return decode(m_translator->trTypedefDocumentation()); }
    QCString trEnumerationTypeDocumentation()  
    { return decode(m_translator->trEnumerationTypeDocumentation()); }
    QCString trFunctionDocumentation()  
    { return decode(m_translator->trFunctionDocumentation()); }
    QCString trVariableDocumentation()  
    { return decode(m_translator->trVariableDocumentation()); }
    QCString trCompounds()  
    { return decode(m_translator->trCompounds()); }
    QCString trGeneratedAt(const char *date,const char *projName)  
    { return decode(m_translator->trGeneratedAt(date,projName)); }
    QCString trWrittenBy()  
    { return decode(m_translator->trWrittenBy()); }
    QCString trClassDiagram(const char *clName)  
    { return decode(m_translator->trClassDiagram(clName)); }
    QCString trForInternalUseOnly()  
    { return decode(m_translator->trForInternalUseOnly()); }
    QCString trWarning()  
    { return decode(m_translator->trWarning()); }
    QCString trVersion()  
    { return decode(m_translator->trVersion()); }
    QCString trDate()  
    { return decode(m_translator->trDate()); }
    QCString trReturns()  
    { return decode(m_translator->trReturns()); }
    QCString trSeeAlso()  
    { return decode(m_translator->trSeeAlso()); }
    QCString trParameters()  
    { return decode(m_translator->trParameters()); }
    QCString trExceptions()  
    { return decode(m_translator->trExceptions()); }
    QCString trGeneratedBy()  
    { return decode(m_translator->trGeneratedBy()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990307
//////////////////////////////////////////////////////////////////////////
    
    QCString trNamespaceList()  
    { return decode(m_translator->trNamespaceList()); }
    QCString trNamespaceListDescription(bool extractAll)  
    { return decode(m_translator->trNamespaceListDescription(extractAll)); }
    QCString trFriends()  
    { return decode(m_translator->trFriends()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 0.49-990405
//////////////////////////////////////////////////////////////////////////
    
    QCString trRelatedFunctionDocumentation()  
    { return decode(m_translator->trRelatedFunctionDocumentation()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 0.49-990425
//////////////////////////////////////////////////////////////////////////

    QCString trCompoundReference(const char *clName,
                                 ClassDef::CompoundType compType,
                                 bool isTemplate)  
    { return decode(m_translator->trCompoundReference(clName,compType,isTemplate)); }

    QCString trFileReference(const char *fileName)  
    { return decode(m_translator->trFileReference(fileName)); }
    QCString trNamespaceReference(const char *namespaceName)  
    { return decode(m_translator->trNamespaceReference(namespaceName)); }
    
    QCString trPublicMembers()  
    { return decode(m_translator->trPublicMembers()); }
    QCString trPublicSlots()  
    { return decode(m_translator->trPublicSlots()); }
    QCString trSignals()  
    { return decode(m_translator->trSignals()); }
    QCString trStaticPublicMembers()  
    { return decode(m_translator->trStaticPublicMembers()); }
    QCString trProtectedMembers()  
    { return decode(m_translator->trProtectedMembers()); }
    QCString trProtectedSlots()  
    { return decode(m_translator->trProtectedSlots()); }
    QCString trStaticProtectedMembers()  
    { return decode(m_translator->trStaticProtectedMembers()); }
    QCString trPrivateMembers()  
    { return decode(m_translator->trPrivateMembers()); }
    QCString trPrivateSlots()  
    { return decode(m_translator->trPrivateSlots()); }
    QCString trStaticPrivateMembers()  
    { return decode(m_translator->trStaticPrivateMembers()); }
    QCString trWriteList(int numEntries)  
    { return decode(m_translator->trWriteList(numEntries)); }
    QCString trInheritsList(int numEntries)  
    { return decode(m_translator->trInheritsList(numEntries)); }
    QCString trInheritedByList(int numEntries)  
    { return decode(m_translator->trInheritedByList(numEntries)); }
    QCString trReimplementedFromList(int numEntries)  
    { return decode(m_translator->trReimplementedFromList(numEntries)); }
    QCString trReimplementedInList(int numEntries)  
    { return decode(m_translator->trReimplementedInList(numEntries)); }
    QCString trNamespaceMembers()  
    { return decode(m_translator->trNamespaceMembers()); }
    QCString trNamespaceMemberDescription(bool extractAll)  
    { return decode(m_translator->trNamespaceMemberDescription(extractAll)); }
    QCString trNamespaceIndex()  
    { return decode(m_translator->trNamespaceIndex()); }
    QCString trNamespaceDocumentation()  
    { return decode(m_translator->trNamespaceDocumentation()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990522
//////////////////////////////////////////////////////////////////////////

    QCString trNamespaces()  
    { return decode(m_translator->trNamespaces()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990728
//////////////////////////////////////////////////////////////////////////

    QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
                                          bool single)  
    { return decode(m_translator->trGeneratedFromFiles(compType,single)); }
    QCString trAlphabeticalList()  
    { return decode(m_translator->trAlphabeticalList()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-990901
//////////////////////////////////////////////////////////////////////////

    QCString trReturnValues()  
    { return decode(m_translator->trReturnValues()); }
    QCString trMainPage()  
    { return decode(m_translator->trMainPage()); }
    QCString trPageAbbreviation()  
    { return decode(m_translator->trPageAbbreviation()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-991003
//////////////////////////////////////////////////////////////////////////

    QCString trDefinedAtLineInSourceFile()  
    { return decode(m_translator->trDefinedAtLineInSourceFile()); }
    QCString trDefinedInSourceFile()  
    { return decode(m_translator->trDefinedInSourceFile()); }

//////////////////////////////////////////////////////////////////////////
// new since 0.49-991205
//////////////////////////////////////////////////////////////////////////

    QCString trDeprecated()  
    { return decode(m_translator->trDeprecated()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.0.0
//////////////////////////////////////////////////////////////////////////

    QCString trCollaborationDiagram(const char *clName)  
    { return decode(m_translator->trCollaborationDiagram(clName)); }
    QCString trInclDepGraph(const char *fName)  
    { return decode(m_translator->trInclDepGraph(fName)); }
    QCString trConstructorDocumentation()  
    { return decode(m_translator->trConstructorDocumentation()); }
    QCString trGotoSourceCode()  
    { return decode(m_translator->trGotoSourceCode()); }
    QCString trGotoDocumentation()  
    { return decode(m_translator->trGotoDocumentation()); }
    QCString trPrecondition()  
    { return decode(m_translator->trPrecondition()); }
    QCString trPostcondition()  
    { return decode(m_translator->trPostcondition()); }
    QCString trInvariant()  
    { return decode(m_translator->trInvariant()); }
    QCString trInitialValue()  
    { return decode(m_translator->trInitialValue()); }
    QCString trCode()  
    { return decode(m_translator->trCode()); }

    QCString trGraphicalHierarchy()  
    { return decode(m_translator->trGraphicalHierarchy()); }
    QCString trGotoGraphicalHierarchy()  
    { return decode(m_translator->trGotoGraphicalHierarchy()); }
    QCString trGotoTextualHierarchy()  
    { return decode(m_translator->trGotoTextualHierarchy()); }
    QCString trPageIndex()  
    { return decode(m_translator->trPageIndex()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.0
//////////////////////////////////////////////////////////////////////////
    
    QCString trNote()  
    { return decode(m_translator->trNote()); }
    QCString trPublicTypes()  
    { return decode(m_translator->trPublicTypes()); }
    QCString trPublicAttribs()  
    { return decode(m_translator->trPublicAttribs()); }
    QCString trStaticPublicAttribs()  
    { return decode(m_translator->trStaticPublicAttribs()); }
    QCString trProtectedTypes()  
    { return decode(m_translator->trProtectedTypes()); }
    QCString trProtectedAttribs()  
    { return decode(m_translator->trProtectedAttribs()); }
    QCString trStaticProtectedAttribs()  
    { return decode(m_translator->trStaticProtectedAttribs()); }
    QCString trPrivateTypes()  
    { return decode(m_translator->trPrivateTypes()); }
    QCString trPrivateAttribs()  
    { return decode(m_translator->trPrivateAttribs()); }
    QCString trStaticPrivateAttribs()  
    { return decode(m_translator->trStaticPrivateAttribs()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.3
//////////////////////////////////////////////////////////////////////////

    QCString trTodo()  
    { return decode(m_translator->trTodo()); }
    QCString trTodoList()  
    { return decode(m_translator->trTodoList()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.1.4
//////////////////////////////////////////////////////////////////////////

    QCString trReferencedBy()  
    { return decode(m_translator->trReferencedBy()); }
    QCString trRemarks()  
    { return decode(m_translator->trRemarks()); }
    QCString trAttention()  
    { return decode(m_translator->trAttention()); }
    QCString trInclByDepGraph()  
    { return decode(m_translator->trInclByDepGraph()); }
    QCString trSince()  
    { return decode(m_translator->trSince()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.1.5
//////////////////////////////////////////////////////////////////////////

    QCString trLegendTitle()  
    { return decode(m_translator->trLegendTitle()); }
    QCString trLegendDocs()  
    { return decode(m_translator->trLegendDocs()); }
    QCString trLegend()  
    { return decode(m_translator->trLegend()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.2.0
//////////////////////////////////////////////////////////////////////////
    
    QCString trTest()  
    { return decode(m_translator->trTest()); }
    QCString trTestList()  
    { return decode(m_translator->trTestList()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.1
//////////////////////////////////////////////////////////////////////////

    QCString trDCOPMethods()  
    { return decode(m_translator->trDCOPMethods()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.2
//////////////////////////////////////////////////////////////////////////

    QCString trProperties()  
    { return decode(m_translator->trProperties()); }
    QCString trPropertyDocumentation()  
    { return decode(m_translator->trPropertyDocumentation()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.4
//////////////////////////////////////////////////////////////////////////

    QCString trClasses()  
    { return decode(m_translator->trClasses()); }
    QCString trPackage(const char *name)  
    { return decode(m_translator->trPackage(name)); }
    QCString trPackageList()  
    { return decode(m_translator->trPackageList()); }
    QCString trPackageListDescription()  
    { return decode(m_translator->trPackageListDescription()); }
    QCString trPackages()  
    { return decode(m_translator->trPackages()); }
    QCString trDefineValue()  
    { return decode(m_translator->trDefineValue()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.2.5
//////////////////////////////////////////////////////////////////////////
    
    QCString trBug()  
    { return decode(m_translator->trBug()); }
    QCString trBugList()  
    { return decode(m_translator->trBugList()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.6
//////////////////////////////////////////////////////////////////////////

    QCString trRTFansicp()  
    { return decode(m_translator->trRTFansicp()); }
    QCString trRTFCharSet()  
    { return decode(m_translator->trRTFCharSet()); }
    QCString trRTFGeneralIndex()  
    { return decode(m_translator->trRTFGeneralIndex()); }
   
    // Translation of the word

    QCString trClass(bool first_capital, bool singular)  
    { return decode(m_translator->trClass(first_capital,singular)); }
    QCString trFile(bool first_capital, bool singular)  
    { return decode(m_translator->trFile(first_capital,singular)); }
    QCString trNamespace(bool first_capital, bool singular)  
    { return decode(m_translator->trNamespace(first_capital,singular)); }
    QCString trGroup(bool first_capital, bool singular)  
    { return decode(m_translator->trGroup(first_capital,singular)); }
    QCString trPage(bool first_capital, bool singular)  
    { return decode(m_translator->trPage(first_capital,singular)); }
    QCString trMember(bool first_capital, bool singular)  
    { return decode(m_translator->trMember(first_capital,singular)); }
    QCString trGlobal(bool first_capital, bool singular)  
    { return decode(m_translator->trGlobal(first_capital,singular)); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.7
//////////////////////////////////////////////////////////////////////////
        
    QCString trAuthor(bool first_capital, bool singular)  
    { return decode(m_translator->trAuthor(first_capital,singular)); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.11
//////////////////////////////////////////////////////////////////////////

    QCString trReferences()  
    { return decode(m_translator->trReferences()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.13
//////////////////////////////////////////////////////////////////////////

    QCString trImplementedFromList(int numEntries)  
    { return decode(m_translator->trImplementedFromList(numEntries)); }
    QCString trImplementedInList(int numEntries)  
    { return decode(m_translator->trImplementedInList(numEntries)); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.2.16
//////////////////////////////////////////////////////////////////////////

    QCString trRTFTableOfContents()  
    { return decode(m_translator->trRTFTableOfContents()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.2.17
//////////////////////////////////////////////////////////////////////////

    QCString trDeprecatedList()  
    { return decode(m_translator->trDeprecatedList()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.2.18
//////////////////////////////////////////////////////////////////////////

    QCString trEvents()  
    { return decode(m_translator->trEvents()); }
    QCString trEventDocumentation()  
    { return decode(m_translator->trEventDocumentation()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.3
//////////////////////////////////////////////////////////////////////////

    QCString trPackageTypes()  
    { return decode(m_translator->trPackageTypes()); }
    QCString trPackageMembers()  
    { return decode(m_translator->trPackageMembers()); }
    QCString trStaticPackageMembers()  
    { return decode(m_translator->trStaticPackageMembers()); }
    QCString trPackageAttribs()  
    { return decode(m_translator->trPackageAttribs()); }
    QCString trStaticPackageAttribs()  
    { return decode(m_translator->trStaticPackageAttribs()); }
    
//////////////////////////////////////////////////////////////////////////
// new since 1.3.1
//////////////////////////////////////////////////////////////////////////

    QCString trAll()  
    { return decode(m_translator->trAll()); }
    QCString trCallGraph()  
    { return decode(m_translator->trCallGraph()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.3.3
//////////////////////////////////////////////////////////////////////////

    QCString trSearchForIndex()  
    { return decode(m_translator->trSearchForIndex()); }
    QCString trSearchResultsTitle()  
    { return decode(m_translator->trSearchResultsTitle()); }
    QCString trSearchResults(int numDocuments)  
    { return decode(m_translator->trSearchResults(numDocuments)); }
    QCString trSearchMatches()  
    { return decode(m_translator->trSearchMatches()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.3.8
//////////////////////////////////////////////////////////////////////////

    QCString trSourceFile(QCString& filename)  
    { return decode(m_translator->trSourceFile(filename)); }

//////////////////////////////////////////////////////////////////////////
// new since 1.3.9
//////////////////////////////////////////////////////////////////////////

    QCString trDirIndex()  
    { return decode(m_translator->trDirIndex()); }
    QCString trDirDocumentation()  
    { return decode(m_translator->trDirDocumentation()); }
    QCString trDirectories()  
    { return decode(m_translator->trDirectories()); }
    QCString trDirDescription()  
    { return decode(m_translator->trDirDescription()); }
    QCString trDirReference(const char *dirName)  
    { return decode(m_translator->trDirReference(dirName)); }
    QCString trDir(bool first_capital, bool singular)  
    { return decode(m_translator->trDir(first_capital,singular)); }

//////////////////////////////////////////////////////////////////////////
// new since 1.4.1
//////////////////////////////////////////////////////////////////////////

    QCString trOverloadText()  
    { return decode(m_translator->trOverloadText()); }

//////////////////////////////////////////////////////////////////////////
// new since 1.4.6
//////////////////////////////////////////////////////////////////////////
    
    QCString trCallerGraph()  
    { return decode(m_translator->trCallerGraph()); }
    QCString trEnumerationValueDocumentation()  
    { return decode(m_translator->trEnumerationValueDocumentation()); }

  private:
    Translator *m_translator;
    iconv_t m_cd;

};

#endif
